import 'package:protobuf3_parser/protobuf3_parser.dart';

class Message3 extends Element3 {
  Message3? parent;
  List<MessageField> fields;
  List<Element3> subElements;

  bool get isCoreType => this is CoreTypeMessage;

  Message3(
    String name, {
    List<Comment> comments = const [],
    this.fields = const [],
    this.subElements = const [],
    this.parent,
  }) : super(name, comments);

  bool get hasParent => parent != null;

  String toProtobufString([int indentLevel = 0]) {
    final indent = "\t" * indentLevel;

    final x = StringBuffer();
    for (final documentation in documentations) {
      x.writeln("$indent${documentation.actualText}");
    }
    x.writeln("${indent}message $name {");

    int iField = 0;
    for (final field in fields) {
      var fieldId = field.id;
      if (fieldId == null) {
        fieldId = ++iField;
      } else {
        iField++;
      }

      for (final fieldDocumentation in field.documentations) {
        x.writeln("\t${fieldDocumentation.actualText}");
      }

      x.write("$indent\t");
      if (field.isList) x.write("repeated ");
      x.writeln("${field.type.name} ${field.name} = $fieldId;");
    }

    int iSub = indentLevel;
    for (final subElement in subElements) {
      if (subElement is Message3) {
        x.writeln(subElement.toProtobufString(++iSub));
      } else if (subElement is Enum3) {
        x.writeln(subElement.toProtobufString(++iSub));
      }
    }

    x.writeln("$indent}");
    return x.toString();
  }

  @override
  String toString() => toProtobufString();
}
