from argparse import ArgumentParser
from genericpath import isdir, isfile
from os.path import dirname, abspath, join, exists, basename
from os import system, walk
from shutil import rmtree, which

JAVA                = which("java")

ANTLR_SEARCH_PATHS  = [
    "/usr/local/li",
]

THIS_DIR            = dirname(abspath(__file__))
ROOT_DIR            = dirname(THIS_DIR)

GRAMMER_FILE        = join(ROOT_DIR, "grammer/Protobuf3.g4")
GEN_DIR             = join(ROOT_DIR, "lib/__generated__")


parser = ArgumentParser()
parser.add_argument("--antlr", help="The antlr4 jar file path or a directory path to search inside.")
args = parser.parse_args()


if exists(GEN_DIR):
    rmtree(GEN_DIR)



def is_antlr_jar(path: str)-> bool:
    file_name = basename(path)
    return file_name.startswith("antlr-4.10.1") and file_name.endswith(".jar")

def search_antlr_in(path: str) -> str | None:
    for root, _, files in walk(path):
        for file_name in files:
            file_path = join(root, file_name)
            if is_antlr_jar(file_path):
                return file_path


antlr_jar           = args.antlr


if antlr_jar:
    if isdir(antlr_jar):
        ANTLR_SEARCH_PATHS.append(antlr_jar)
        antlr_jar = search_antlr_in(antlr_jar)


if not antlr_jar:
    for path in ANTLR_SEARCH_PATHS:
        x = search_antlr_in(path)
        if x:
            antlr_jar = x
            break

if not antlr_jar:
    print("Couldn't find antlr in:")
    for path in ANTLR_SEARCH_PATHS:
        print("-", path)
        exit(1)

print(f"using {antlr_jar} ..")

ANTLR               = f"{JAVA} -Xmx500M -cp '{antlr_jar}:$CLASSPATH' org.antlr.v4.Tool"

exit(system(f"{ANTLR} {GRAMMER_FILE} -Dlanguage=Dart -o {GEN_DIR}"))